@@ -90,6 +90,7 @@ public class ProjectGenerator implements IntegrationProjectGenerator {
     private final Mustache applicationPropertiesMustache;
     private final Mustache restRoutesMustache;
     private final Mustache pomMustache;
+    private final Mustache settingsXmlMustache;
     private final MavenProperties mavenProperties;
 
     public ProjectGenerator(ProjectGeneratorConfiguration configuration, IntegrationResourceManager resourceManager, MavenProperties mavenProperties) throws IOException {
@@ -103,6 +104,7 @@ public class ProjectGenerator implements IntegrationProjectGenerator {
         this.applicationPropertiesMustache = compile(mf, configuration, "application.properties.mustache", "application.properties");
         this.restRoutesMustache = compile(mf, configuration, "RestRouteConfiguration.java.mustache", "RestRouteConfiguration.java");
         this.pomMustache = compile(mf, configuration, "pom.xml.mustache", "pom.xml");
+        settingsXmlMustache = compile(mf, configuration, "settings.xml.mustache", "settings.xml");
     }
 
     @Override
@@ -173,6 +175,12 @@ public class ProjectGenerator implements IntegrationProjectGenerator {
                 final String componentScheme = maybeComponentScheme.get();
                 final Map<String, ConfigurationProperty> configurationProperties = CollectionsUtils.aggregate(connector.getProperties(),
                     action.getProperties());
+                final Map<String, String> configuredProperties = CollectionsUtils.aggregate(
+                    actionDescriptor.getConfiguredProperties(), // 1. action
+                    step.getConfiguredProperties(), // 2. step
+                    connection.getConfiguredProperties(), // 3. connection
+                    connector.getConfiguredProperties() // 4. connector
+                );
 
                 // Workaround for
                 // https://github.com/syndesisio/syndesis/issues/1713
@@ -180,11 +188,19 @@ public class ProjectGenerator implements IntegrationProjectGenerator {
                     final String propertyName = entry.getKey();
                     final ConfigurationProperty configurationProperty = entry.getValue();
 
-                    final String defaultValue = Objects.toString(configurationProperty.getDefaultValue(), null);
                     boolean isSecret = connector.isSecret(propertyName) || action.isSecret(propertyName);
 
-                    if (Strings.isEmptyOrBlank(defaultValue) && isSecret) {
+                    if (!isSecret) {
+                        continue;
+                    }
+
+                    final String defaultValue = Objects.toString(configurationProperty.getDefaultValue(), null);
+                    final String configuredValue = configuredProperties.get(propertyName);
+
+                    if (Strings.isEmptyOrBlank(configuredValue)) {
                         addDecryptedKeyProperty(resourceManager, properties, flowIndex, stepIndex, componentScheme, propertyName, defaultValue);
+                    } else {
+                        addDecryptedKeyProperty(resourceManager, properties, flowIndex, stepIndex, componentScheme, propertyName, configuredValue);
                     }
                 }
 
@@ -218,7 +234,7 @@ public class ProjectGenerator implements IntegrationProjectGenerator {
 
         return ProjectGeneratorHelper.generate(
             new PomContext(
-            	Optional.ofNullable(configuration.getArtifactId()).orElse("project"),
+                Optional.ofNullable(configuration.getArtifactId()).orElse("project"),
                 integration.getName(),
                 integration.getDescription().orElse(null),
                 dependencies,
@@ -227,11 +243,17 @@ public class ProjectGenerator implements IntegrationProjectGenerator {
         );
     }
 
+    public byte[] generateSettingsXml() throws IOException {
+        return ProjectGeneratorHelper.generate(mavenProperties, settingsXmlMustache);
+    }
+
     private static void addDecryptedKeyProperty(IntegrationResourceManager resourceManager, Properties properties, int flowIndex, int stepIndex, String propKeyPrefix, String propertyKey, String propertyVal) {
         String key = String.format("flow-%d.%s-%d.%s", flowIndex, propKeyPrefix, stepIndex, propertyKey);
         String val = mandatoryDecrypt(resourceManager, propertyKey, propertyVal);
 
-        properties.put(key, val);
+        if (val != null) {
+            properties.put(key, val);
+        }
     }
 
     private void addAdditionalResources(TarArchiveOutputStream tos) throws IOException {
@@ -300,7 +322,7 @@ public class ProjectGenerator implements IntegrationProjectGenerator {
 
                 addResource(tos, ".s2i/bin/assemble", "s2i/assemble");
                 addResource(tos, "prometheus-config.yml", "templates/prometheus-config.yml");
-                addResource(tos, "configuration/settings.xml", "templates/settings.xml");
+                addTarEntry(tos, "configuration/settings.xml", generateSettingsXml());
 
                 addAdditionalResources(tos);
 
@@ -324,15 +346,15 @@ public class ProjectGenerator implements IntegrationProjectGenerator {
             addTarEntry(tos, "src/main/resources/loader.properties", generateExtensionLoader(extensions));
 
             for (String extensionId : extensions) {
-                addTarEntry(
-                    tos,
-                    "extensions/" + Names.sanitize(extensionId) + ".jar",
-                    IOUtils.toByteArray(
-                        resourceManager.loadExtensionBLOB(extensionId).orElseThrow(
-                            () -> new IllegalStateException("No extension blob for extension with id:" + extensionId)
-                        )
-                    )
-                );
+                try (InputStream extension = resourceManager.loadExtensionBLOB(extensionId).orElseThrow(
+                    () -> new IllegalStateException("No extension blob for extension with id:" + extensionId)
+                )) {
+                    addTarEntry(
+                        tos,
+                        "extensions/" + Names.sanitize(extensionId) + ".jar",
+                        IOUtils.toByteArray(extension)
+                    );
+                }
             }
         }
     }
